"""Tests for cpu_info module."""

import unittest
import unittest.mock

from system_query.available_features import CPU
import system_query.cpu_info
from system_query.cpu_info import query_cpu_clock, query_cpu_cores, _get_cache_size


class Tests(unittest.TestCase):

    @unittest.skipIf(not CPU, 'skipping CPU cache query')
    def test_cpu_cache_size(self):
        info = _get_cache_size(1, {'l1_cache_size': '512'})
        self.assertIsInstance(info, int)
        self.assertEqual(info, 512 * 1024)

    @unittest.skipIf(not CPU, 'skipping CPU cache query')
    def test_cpu_cache_size_with_units(self):
        info = _get_cache_size(1, {'l1_cache_size': '512 kB'})
        self.assertIsInstance(info, int)
        self.assertEqual(info, 512 * 1000)
        info = _get_cache_size(1, {'l1_cache_size': '512 KB'})
        self.assertIsInstance(info, int)
        self.assertEqual(info, 512 * 1024)
        info = _get_cache_size(1, {'l1_cache_size': '512 KiB'})
        self.assertIsInstance(info, int)
        self.assertEqual(info, 512 * 1024)
        info = _get_cache_size(1, {'l1_cache_size': '2 MB'})
        self.assertIsInstance(info, int)
        self.assertEqual(info, 2 * 1024 ** 2)
        info = _get_cache_size(1, {'l1_cache_size': '4 MiB'})
        self.assertIsInstance(info, int)
        self.assertEqual(info, 4 * 1024 ** 2)

    @unittest.skipIf(not CPU, 'skipping CPU cache query')
    def test_cpu_cache_size_with_units_and_instances(self):
        info = _get_cache_size(1, {'l1_cache_size': '192 KiB (6 instances)'})
        self.assertIsInstance(info, int)
        self.assertEqual(info, 192 * 1024)

    @unittest.skipIf(not CPU, 'skipping CPU cache query')
    def test_query_cpu_clock_unsupported(self):
        with unittest.mock.patch.object(system_query.cpu_info, 'CPU_CLOCK', False):
            self.assertEqual(query_cpu_clock(), (None, None, None))

    @unittest.skipIf(not CPU, 'skipping CPU cache query')
    def test_query_cpu_cores_unsupported(self):
        with unittest.mock.patch.object(system_query.cpu_info, 'CPU_CORES', False):
            self.assertEqual(query_cpu_cores(), (None, None))



"""Initialization of tests for system_query package."""

import logging

from system_query.__main__ import Logging


class TestsLogging(Logging):
    """Test logging configuration."""

    level_global = logging.DEBUG


TestsLogging.configure()



"""Tests for all_info module."""

import collections.abc
import platform
import unittest

from system_query.all_info import query_all


class IntegrationChecks(unittest.TestCase):
    """Check all info modules."""

    def query_all_and_check(self, sudo: bool):
        """Check that query results contain expected entries."""
        info = query_all(sudo=sudo)
        self.assertIsInstance(info, collections.abc.Mapping)
        self.assertGreater(len(info), 1)
        self.assertIn('host', info)
        self.assertIn('os', info)
        self.assertIn('cpu', info)
        self.assertIn('ram', info)
        if not sudo:
            return
        min_length = 1 if platform.system() == 'Linux' else 0
        ram_banks = info.get('ram', {}).get('banks', [])
        self.assertGreaterEqual(len(ram_banks), min_length)
        for ram_bank in ram_banks:
            self.assertIsInstance(ram_bank, dict)


class Tests(IntegrationChecks):

    def test_query_all(self):
        self.query_all_and_check(False)



"""Functions to query system's hard drives."""

import itertools
import logging
import typing as t

from .available_features import pyudev, HDD

IGNORED_DEVICE_PATHS = {'/dev/bus', '/dev/dm', '/dev/loop', '/dev/md', '/dev/sr'}

_LOG = logging.getLogger(__name__)


def query_hdds() -> t.Dict[str, dict]:
    """Get information about all hard drives."""
    if not HDD:
        return {}
    context = pyudev.Context()
    hdds = {}
    for device in context.list_devices(subsystem='block', DEVTYPE='disk'):
        if device.device_node is None or any(
                device.device_node.startswith(_) for _ in IGNORED_DEVICE_PATHS):
            continue
        hdd = {'size': device.attributes.asint('size')}
        for device_ in itertools.chain([device], device.ancestors):
            try:
                hdd['model'] = device_.attributes.asstring('model').strip()
            except KeyError:
                _LOG.debug('device %s has no attribute "model"', device_.device_path)
                continue
            else:
                _LOG.debug(
                    'got model "%s" for device %s from device %s',
                    hdd['model'], device.device_path, device_.device_path)
            break
        hdds[device.device_node] = hdd
    return hdds
